@@ -52,9 +52,9 @@ def _get_converter():
     parser.add_argument('-r', '--replaceimports', action='store_true',
                         help='replace imports with their used ttypes')
     parser.add_argument('-i', '--indent', type=int, default=2,
-                        help='indent (0-32; default 2)')
+                        help='indent (0-8; default 2)')
     parser.add_argument('-w', '--wrapwidth', type=int, default=96,
-                        help='wrapwidth (0 or 40-240; default 96)')
+                        help='wrapwidth (40-240; default 96)')
     parser.add_argument('-v', '--verbose', action='store_true')
     parser.add_argument(
         '-f', '--fieldnames', action='store_true',
@@ -508,7 +508,7 @@ def _uxf_to_xml(uxo, outfile, indent):
     if uxo.custom:
         root.setAttribute('custom', uxo.custom)
     if uxo.comment:
-        root.setAttribute('comment', uxo.comment)
+        _xml_add_comment(tree, root, uxo.comment)
     if uxo.imports:
         _xml_add_imports(tree, root, uxo.import_filenames)
     if uxo.tclasses:
@@ -518,6 +518,16 @@ def _uxf_to_xml(uxo, outfile, indent):
         file.write(tree.toprettyxml(indent=indent))
 
 
+def _xml_add_comment(tree, root, comment):
+    element = tree.createElement('comment')
+    if '\n' in comment and ']]>' not in comment:
+        text_element = tree.createCDATASection(comment)
+    else:
+        text_element = tree.createTextNode(comment)
+    element.appendChild(text_element)
+    root.appendChild(element)
+
+
 def _xml_add_imports(tree, root, import_filenames):
     imports_element = tree.createElement('imports')
     for filename in import_filenames: # don't sort!
@@ -533,7 +543,7 @@ def _xml_add_tclasses(tree, root, tclasses):
         tclass_element = tree.createElement('ttype')
         tclass_element.setAttribute('name', tclass.ttype)
         if tclass.comment:
-            tclass_element.setAttribute('comment', tclass.comment)
+            _xml_add_comment(tree, tclass_element, tclass.comment)
         for field in tclass.fields:
             field_element = tree.createElement('field')
             field_element.setAttribute('name', field.name)
@@ -565,7 +575,7 @@ def _xml_add_list(tree, root, lst, *, tag='list'):
         list_element.setAttribute('vtype', vtype)
     comment = getattr(lst, 'comment', None)
     if comment is not None:
-        list_element.setAttribute('comment', comment)
+        _xml_add_comment(tree, list_element, comment)
     for value in lst:
         _xml_add_value(tree, list_element, value)
     root.appendChild(list_element)
@@ -581,7 +591,7 @@ def _xml_add_map(tree, root, map):
         map_element.setAttribute('vtype', vtype)
     comment = getattr(map, 'comment', None)
     if comment is not None:
-        map_element.setAttribute('comment', comment)
+        _xml_add_comment(tree, map_element, comment)
     for key, value in map.items():
         key_element = tree.createElement('key')
         _xml_add_value(tree, key_element, key)
@@ -596,7 +606,7 @@ def _xml_add_table(tree, root, table):
     table_element = tree.createElement('table')
     table_element.setAttribute('name', table.ttype)
     if table.comment is not None:
-        table_element.setAttribute('comment', table.comment)
+        _xml_add_comment(tree, table_element, table.comment)
     for value in table:
         _xml_add_value(tree, table_element, value)
     root.appendChild(table_element)
@@ -661,8 +671,11 @@ class _UxfSaxHandler(xml.sax.handler.ContentHandler):
         self.stack = None
         self.tclass = None
         self.imports_list = []
+        self.incomment = False
         self.instr = False
         self.inbytes = False
+        self.intclass = False
+        self.comment = ''
         self.string = ''
         self.bytes = ''
         self.uxo = uxf.Uxf()
@@ -672,30 +685,30 @@ class _UxfSaxHandler(xml.sax.handler.ContentHandler):
         d = {key: value for key, value in attributes.items()}
         if name == 'uxf':
             self.uxo.custom = d.get('custom', '')
-            self.uxo.comment = d.get('comment')
         elif name in {'imports', 'ttypes'}:
             pass
         elif name == 'import':
             self.imports_list.append(d['filename'])
         elif name == 'ttype':
-            self.tclass = uxf.TClassBuilder(d['name'],
-                                            comment=d.get('comment'))
+            self.tclass = uxf.TClassBuilder(d['name'])
+            self.intclass = True
         elif name == 'field':
             self.tclass.append(uxf.Field(d['name'], d.get('vtype')))
         elif name == 'map':
-            container = uxf.Map(ktype=d.get('ktype'), vtype=d.get('vtype'),
-                                comment=d.get('comment'))
+            container = uxf.Map(ktype=d.get('ktype'), vtype=d.get('vtype'))
             self.start_container(container)
         elif name == 'list':
-            container = uxf.List(vtype=d.get('vtype'),
-                                 comment=d.get('comment'))
+            container = uxf.List(vtype=d.get('vtype'))
             self.start_container(container)
         elif name == 'table':
             tclass = self.uxo.tclasses[d['name']]
-            container = uxf.Table(tclass, comment=d.get('comment'))
+            container = uxf.Table(tclass)
             self.start_container(container)
         elif name in {'key', 'value'}:
             pass # container.append() doesn't need this distinction
+        elif name == 'comment':
+            self.incomment = True
+            self.comment = ''
         elif name == 'str':
             self.instr = True
             self.string = ''
@@ -728,8 +741,18 @@ class _UxfSaxHandler(xml.sax.handler.ContentHandler):
         elif name == 'ttype':
             self.uxo.tclasses[self.tclass.ttype] = self.tclass.build()
             self.tclass = None
+            self.intclass = False
         elif name in {'map', 'list', 'table'}:
             self.stack.pop()
+        elif name == 'comment':
+            if self.intclass:
+                self.tclass.comment = self.comment
+            elif self.stack:
+                self.stack[-1].comment = self.comment
+            else:
+                self.uxo.comment = self.comment
+            self.comment = ''
+            self.incomment = False
         elif name == 'str':
             _append_to_parent(self.stack, self.string)
             self.string = ''
@@ -745,6 +768,8 @@ class _UxfSaxHandler(xml.sax.handler.ContentHandler):
             self.bytes += text
         elif self.instr:
             self.string += text
+        elif self.incomment:
+            self.comment += text
 
 
     def start_container(self, container):
@@ -827,9 +852,9 @@ Use -d or --dropunused to drop unused ttype definitions and imports.
 Use -r or --replaceimports to replace imports with ttype definitions
 to make the outfile standalone (i.e., not dependent on any imports).
 
-Indent defaults to 2 and accepts a range of 0-32.
+Indent defaults to 2 and accepts a range of 0-8.
 
-Wrapwidth defaults to 96 and accepts 0 (no wrapping) or a range of 40-240.
+Wrapwidth defaults to 96 and accepts a range of 40-240.
 
 For indent and wrapwidth the default is silently used if an out of range
 value is given. Indent applies when converting to JSON, UXF, and XML;
